import numpy as np
import yfinance as yf
from datetime import datetime, timedelta

def calculate_returns(data):
    data["Daily Returns"] = data["Adj Close"].pct_change()
    data["Cumulative Returns"] = (1 + data["Daily Returns"]).cumprod()
    return data

def alpha(data, market = "^NSEI", interval = "1d", beta = 0.8, risk = 0.02):
    market_df = yf.download(market, start = data.index.min(), end = data.index.max(), interval = interval, progress = False)
    market_df['Daily Return'] = market_df['Adj Close'].pct_change()
    data['Daily Returns'] = data['Adj Close'].pct_change()
    
    # Capital Asset Pricing Model (CAPM)
    expected_return = risk + (beta * (market_df['Daily Return'].mean() - risk))

    alpha = data['Daily Returns'].mean() - expected_return
    print(f"Alpha using {market} as a proxy for the market is: {round(alpha, 3)}")


def beta(data, market = "^NSEI", interval = "1d"):
    # Nifty 50 as a proxy for the market
    market_df = yf.download(market, start = data.index.min(), end = data.index.max(), interval = interval, progress = False)
    market_df['Daily Return'] = market_df['Adj Close'].pct_change()

    covariance = data['Adj Close'].pct_change().cov(market_df['Daily Return'])
    variance_market = market_df['Daily Return'].var()

    beta = covariance / variance_market
    print(f"Beta using {market} as a proxy for the market is: {round(beta, 3)}")

def volatility(data, trading_days = 252):
    data['Daily Returns'] = data['Adj Close'].pct_change()
    data_volatility = data['Daily Returns'].std()
    annualized_volatility = data_volatility * (trading_days**0.5)
    
    print(f"Standard deviation: {round(data_volatility, 3)}")
    print(f"Annualized volatility: {round(annualized_volatility, 3)}")

def sharpe(data, risk = 0.02, trading_days = 252):
    average_return = data['Daily Returns'].mean() * trading_days
    volatility = data['Daily Returns'].std() * (trading_days**0.5)

    sharpe_ratio = (average_return - risk) / volatility
    print(f"Sharpe Ratio: {round(sharpe_ratio, 3)}")


def treynor(data, beta = 1.2, risk = 0.02, trading_days = 252):
    average_return = data['Daily Returns'].mean() * trading_days

    treynor_ratio = (average_return - risk) / beta
    print(f"Treynor Ratio: {round(treynor_ratio, 3)}")

def info(data, market = "^NSEI", interval = "1d", risk = 0.02, trading_days = 252):
    market_df = yf.download(market, start = data.index.min(), end = data.index.max(), interval = interval, progress = False)
    market_df['Daily Return'] = market_df['Adj Close'].pct_change()
    portfolio_returns = data['Daily Returns']
    benchmark_returns = market_df['Daily Return']
    portfolio_excess_return = (portfolio_returns - risk).mean() * trading_days
    benchmark_excess_return = (benchmark_returns - risk).mean() * trading_days

    tracking_error = (portfolio_returns - benchmark_returns).std() * (trading_days**0.5)
    information_ratio = (portfolio_excess_return - benchmark_excess_return) / tracking_error

    print(f"Information Ratio: {round(information_ratio, 3)}")

def sortino(data, risk = 0.02, trading_days = 252):
    portfolio_returns = data['Daily Returns']
    portfolio_excess_return = (portfolio_returns - risk).mean() * trading_days
    downside_returns = np.minimum(portfolio_returns - risk, 0)
    downside_semi_variance = (downside_returns**2).sum() / len(downside_returns)

    sortino_ratio = portfolio_excess_return / (downside_semi_variance**0.5)
    print(f"Sortino Ratio: {round(sortino_ratio, 3)}")




import yfinance as yf
from datetime import datetime, timedelta

def get_data(ticker, start_date, end_date = datetime.today().strftime("%Y-%m-%d"), interval = "1d"):
    data = yf.download(ticker, start = start_date, end = end_date, interval = interval, progress = False)
    return data




import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.dates as mdates

# SMA
def sma(stock_data, sma_period = 15):
  stock_data["SMA"] = stock_data["Adj Close"].rolling(window = sma_period).mean()
  return stock_data

# EMA
def ema(stock_data, ema_period = 15, adjust = False):
  stock_data["EMA"] = stock_data["Adj Close"].ewm(span = ema_period, adjust = adjust).mean()
  return stock_data

# RSI
def rsi(stock_data, rsi_period = 14):
  delta = stock_data['Adj Close'].diff()
  gain = delta.where(delta > 0, 0)
  loss = -delta.where(delta < 0, 0)
  avg_gain = gain.rolling(window = rsi_period).mean()
  avg_loss = loss.rolling(window = rsi_period).mean()
  rs = avg_gain / avg_loss
  stock_data["RSI"] = 100 - (100 / (1 + rs))
  return stock_data

# Connors RSI
def connors_rsi(stock_data, rsi_period = 14, updown_length = 2):
  delta = stock_data['Adj Close'].diff()
  gain = delta.where(delta > 0, 0)
  loss = -delta.where(delta < 0, 0)
  avg_gain = gain.rolling(window = rsi_period).mean()
  avg_loss = loss.rolling(window = rsi_period).mean()
  rs = avg_gain / avg_loss
  stock_data["RSI"] = 100 - (100 / (1 + rs))

  stock_data['Updown'] = stock_data['Adj Close'].diff(updown_length)
  stock_data['Updown'] = stock_data['Updown'].apply(lambda x: 1 if x > 0 else (-1 if x < 0 else 0))

  stock_data['Connors_RSI'] = (stock_data['RSI'] + avg_gain + stock_data['Updown']) / 3
  return stock_data

# Stochastic Oscillator
def stoch(stock_data, k = 14, d = 3):
  stock_data["Lowest Low"] = stock_data["Low"].rolling(window = k).min()
  stock_data["Highest High"] = stock_data["High"].rolling(window = k).max()
  stock_data["%K"] = ((stock_data["Adj Close"] - stock_data["Lowest Low"]) / (stock_data["Highest High"] - stock_data["Lowest Low"])) * 100
  stock_data["%D"] = stock_data["%K"].rolling(window = d).mean()
  return stock_data

# Bollinger Bands
def bollinger_bands(stock_data, sma_period = 20, std_dev = 2):
  stock_data["SMA - BB"] = stock_data["Adj Close"].rolling(window = sma_period).mean()
  stock_data["Std Dev - BB"] = stock_data["Adj Close"].rolling(window = sma_period).std()
  stock_data["Upper Band"] = stock_data["SMA - BB"] + (stock_data["Std Dev - BB"] * std_dev)
  stock_data["Lower Band"] = stock_data["SMA - BB"] - (stock_data["Std Dev - BB"] * std_dev)
  return stock_data

# Aroon Oscillator
def aroon(stock_data, aroon_period = 25):
  stock_data["Aroon_High"] = stock_data["High"].rolling(window = aroon_period).apply(lambda x: x.argmax() + 1)
  stock_data["Aroon_Low"] = stock_data["Low"].rolling(window = aroon_period).apply(lambda x: x.argmin() + 1)
  stock_data["Aroon_Up"] = ((aroon_period - stock_data["Aroon_High"]) / aroon_period) * 100
  stock_data["Aroon_Down"] = ((aroon_period - stock_data["Aroon_Low"]) / aroon_period) * 100
  stock_data["Aroon Oscillator"] = stock_data["Aroon_Up"] - stock_data["Aroon_Down"]
  return stock_data

# Moving Average Convergence Divergence
def macd(stock_data, short_period = 12, long_period = 26, signal = 9, adjust = False):
  stock_data['Short_EMA'] = stock_data['Adj Close'].ewm(span = short_period, adjust = adjust).mean()
  stock_data['Long_EMA'] = stock_data['Adj Close'].ewm(span = long_period, adjust = adjust).mean()
  stock_data['MACD_Line'] = stock_data['Short_EMA'] - stock_data['Long_EMA']
  stock_data['Signal_Line'] = stock_data['MACD_Line'].ewm(span = signal, adjust = adjust).mean()

  stock_data['MACD_Histogram'] = stock_data['MACD_Line'] - stock_data['Signal_Line']
  return stock_data

# Ichimoku Cloud
def ichimoku_cloud(stock_data, conversion = 9, base = 26, lead_b = 52):
  stock_data['Tenkan-sen'] = (stock_data['High'].rolling(window = conversion).max() + stock_data['Low'].rolling(window = conversion).min()) / 2
  stock_data['Kijun-sen'] = (stock_data['High'].rolling(window = base).max() + stock_data['Low'].rolling(window = base).min()) / 2
  stock_data['Senkou Span A'] = ((stock_data['Tenkan-sen'] + stock_data['Kijun-sen']) / 2).shift(base)
  stock_data['Senkou Span B'] = ((stock_data['High'].rolling(window = lead_b).max() + stock_data['Low'].rolling(window = lead_b).min()) / 2).shift(base)
  stock_data['Chikou Span'] = stock_data['Adj Close'].shift(-base)
  return stock_data






from setuptools import setup, find_packages
import codecs
import os

VERSION = '1.0.0'
DESCRIPTION = 'Making Finance easy in Python!'
with open("pyfin_trading/README.md", "r") as f:
    LONG_DESCRIPTION = f.read()

# Setting up
setup(
    name="pyfin_trading",
    version=VERSION,
    author="Poonam Deshmukh",
    author_email="poonamdeshmukh616@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=[],
    keywords=['python', 'finance', 'algorithmic trading', 'technical indicators'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)